﻿// Disclaimer:
// This software is provided AS IS for educational purposes only,
// with no warranties, and confers no rights. Error checking is 
// elided for clarity. Carry out sufficient tests prior to using
// it in a productive environment.
// CASIO Europe GmbH accepts no responsibility for loss or damage
// arising from its use, including damage from virus.
//
// Please, note: This configuration information may be changed
// without any further notice.
// Casio Europe GmbH does not guarantee future compatibility. By 
// using this software you agree to these terms and conditions.
//
// This file is part of Printer Demo.
//
// Version 1.00 - May 03, 2012
// Supported Devices: IT-9000
//
// Remarks: You need latest versions of patches and runtime
// libraries be installed onto your mobile device.
//////////////////////////////////////////////////////////////////////

using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Collections;
using System.Drawing.Imaging;

[assembly: CLSCompliant(true)]
namespace CasioEurope.Samples.DataCapture
{
    /// <summary>
    /// The SignBox class provides a control that can be used to draw a signature.
    /// </summary>
    public partial class SignBox : Control
    {
        private int x1;
        private int y1;
        private Bitmap signature;
        private Graphics signatureGraphics;
        private Pen signPen;
        private List<SignatureSegment> signatureSegments;

        private static readonly Color BackColorDefault = Color.White;
        private static readonly Color ForeColorDefault = Color.Gainsboro;
        private static readonly Color PenColorDefault = Color.Black;
        private const float PenWidthDefault = 1.0F;

        /// <summary>
        /// Initializes a new instance of the <see cref="SignBox"/> class.
        /// </summary>
        public SignBox()
        {
            InitializeComponent();
            InitControl();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="SignBox"/> class.
        /// </summary>
        /// <param name="container">The container.</param>
        public SignBox(IContainer container)
        {
            container.Add(this);
            InitializeComponent();
            InitControl();
        }

        private void InitControl()
        {
            if (Environment.OSVersion.Platform == PlatformID.WinCE)
            {
                BackColor = BackColorDefault;
                ForeColor = ForeColorDefault;
                Font = new Font(FontFamily.GenericSansSerif, 20, FontStyle.Regular);
            }

            this.signatureSegments = new List<SignatureSegment>();
            this.signPen = new Pen(PenColorDefault, PenWidthDefault);
        }

        /// <summary>
        /// Resets the the contents of the signature box to default settings.
        /// </summary>
        public void Clear()
        {
            if (this.signatureSegments != null)
            {
                this.signatureSegments.Clear();
            }

            InitBitmap();
            Invalidate();
        }

        private void InitBitmap()
        {
            if (this.signature != null)
            {
                this.signature.Dispose();
                this.signatureGraphics.Dispose();
            }

            this.signature = new Bitmap(Size.Width, Size.Height);
            this.signatureGraphics = Graphics.FromImage(this.signature);
            this.signatureGraphics.Clear(BackColor);

            SolidBrush brush = new SolidBrush(ForeColor);
            Size measuredSize = signatureGraphics.MeasureString(Text, Font).ToSize();
            float posX;
            float posY;
            if (Size.Width < measuredSize.Width)
            {
                posX = 0;
            }
            else
            {
                posX = (Size.Width - measuredSize.Width) / 2;
            }
            if (Size.Height < measuredSize.Height)
            {
                posY = 0;
            }
            else
            {
                posY = (Size.Height - measuredSize.Height) / 2;
            }

            this.signatureGraphics.DrawString(
                Text,
                Font,
                brush,
                posX,
                posY);

            brush.Dispose();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            e.Graphics.DrawImage(this.signature, 0, 0);

            base.OnPaint(e);
        }

        protected override void OnMouseDown(MouseEventArgs e)
        {
            this.x1 = e.X;
            this.y1 = e.Y;

            base.OnMouseDown(e);
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (Capture == true &&
                e.X <= this.Size.Width &&
                e.X >= 0 &&
                e.Y <= this.Size.Height &&
                e.Y >= 0)
            {
                if (this.x1 >= 0 && this.y1 >= 0)
                {
                    DrawLines(this.x1, this.y1, e.X, e.Y);
                }
                this.x1 = e.X;
                this.y1 = e.Y;
            }
            else
            {
                this.x1 = -1;
                this.y1 = -1;
            }

            base.OnMouseMove(e);
        }

        /// <summary>
        /// Raises the <see cref="E:System.Windows.Forms.Control.Resize"/> event.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> that contains the event data.</param>
        /// <remarks>Resets the the contents of the signature box to default settings.</remarks>
        protected override void OnResize(EventArgs e)
        {
            Clear();

            base.OnResize(e);
        }

        /// <summary>
        /// Gets or sets the color of the background.
        /// </summary>
        /// <value>
        /// The <see cref="T:System.Drawing.Color"/> of the background.
        /// </value>
        /// <remarks>Changing the value resets the the contents of the signature box to default settings.</remarks>
        public override Color BackColor
        {
            get
            {
                return base.BackColor;
            }
            set
            {
                if (BackColor != value)
                {
                    base.BackColor = value;
                    Clear();
                }
            }
        }

        /// <summary>
        /// Gets or sets the color of the text that appears in the background of the signature box.
        /// </summary>
        /// <value>
        /// The <see cref="T:System.Drawing.Color"/> of the text.
        /// </value>
        /// <remarks>Changing the value resets the the contents of the signature box to default settings.</remarks>
        public override Color ForeColor
        {
            get
            {
                return base.ForeColor;
            }
            set
            {
                if (ForeColor != value)
                {
                    base.ForeColor = value;
                    Clear();
                }
            }
        }

        /// <summary>
        /// Gets or sets the font of the text that appears in the background of the signature box.
        /// </summary>
        /// <value>
        /// The font.
        /// </value>
        /// <remarks>Changing the value resets the the contents of the signature box to default settings.</remarks>
        public override Font Font
        {
            get
            {
                return base.Font;
            }
            set
            {
                if (Font != value)
                {
                    base.Font = value;
                    Clear();
                }
            }
        }

        /// <summary>
        /// Gets or sets the text that appears in the background of the signature box.
        /// </summary>
        /// <value>
        /// The text.
        /// </value>
        /// <remarks>Changing the value resets the the contents of the signature box to default settings.</remarks>
        public override string Text
        {
            get
            {
                return base.Text;
            }
            set
            {
                if (Text != value)
                {
                    base.Text = value;
                    Clear();
                }
            }
        }

        private bool ShouldSerializePenColor()
        {
            return (this.PenColor != PenColorDefault);
        }
        private void ResetPenColor()
        {
            PenColor = PenColorDefault;
        }
        /// <summary>
        /// Gets or sets the color of the pen to draw the signature.
        /// </summary>
        /// <value>
        /// The <see cref="T:System.Drawing.Color"/> of the pen.
        /// </value>
        /// <remarks>Changing the value resets the the contents of the signature box to default settings.</remarks>
        public Color PenColor
        {
            get
            {
                return this.signPen.Color;
            }
            set
            {
                Color color;
                float width;

                if (this.signPen != null)
                {
                    color = this.signPen.Color;
                    width = this.signPen.Width;
                }
                else
                {
                    color = PenColorDefault;
                    width = PenWidthDefault;
                }

                if (color != value)
                {
                    if (this.signPen != null)
                    {
                        this.signPen.Dispose();
                    }
                    this.signPen = new Pen(value, width);
                    Clear();
                }
            }
        }

        private bool ShouldSerializePenWidth()
        {
            return (PenWidth != PenWidthDefault);
        }
        private void ResetPenWidth()
        {
            PenWidth = PenWidthDefault;
        }
        /// <summary>
        /// Gets or sets the width of the pen to draw the signature.
        /// </summary>
        /// <value>
        /// The width of the pen.
        /// </value>
        /// <remarks>Changing the value resets the the contents of the signature box to default settings.</remarks>
        public Single PenWidth
        {
            get
            {
                return this.signPen.Width;
            }
            set
            {
                Color color;
                float width;

                if (this.signPen != null)
                {
                    color = this.signPen.Color;
                    width = this.signPen.Width;
                }
                else
                {
                    color = PenColorDefault;
                    width = PenWidthDefault;
                }

                if (width != value)
                {
                    if (this.signPen != null)
                    {
                        this.signPen.Dispose();
                    }
                    this.signPen = new Pen(color, value);
                    Clear();
                }
            }
        }

        private void DrawLines(int x1, int y1, int x2, int y2)
        {
            using (Graphics g = CreateGraphics())
            {
                g.DrawLine(this.signPen, x1, y1, x2, y2);
            }

            this.signatureGraphics.DrawLine(this.signPen, x1, y1, x2, y2);
            this.signatureSegments.Add(new SignatureSegment(x1, y1, x2, y2));
        }

        /// <summary>
        /// Saves the signature without background text.
        /// </summary>
        /// <param name="filename">The filename.</param>
        /// <param name="format">The format.</param>
        public void Save(string filename, ImageFormat format)        
        {
            using (Bitmap signatureToSave = new Bitmap(Size.Width, Size.Height))
            {
                using (Graphics signatureToSaveGraphics = Graphics.FromImage(signatureToSave))
                {
                    signatureToSaveGraphics.Clear(this.BackColor);

                    foreach (SignatureSegment segment in this.signatureSegments)
                    {
                        signatureToSaveGraphics.DrawLine(
                            this.signPen,
                            segment.X1,
                            segment.Y1,
                            segment.X2,
                            segment.Y2);
                    }
                }

                signatureToSave.Save(filename, format);
            }
        }
    }
}
